// BigIntegerFactorialDriver.java tests the factorial() method

import ann.easyio.*;                            // Screen, Keyboard
import java.math.BigInteger;

class BigIntegerFactorialDriver
{
  final static BigInteger ZERO = new BigInteger("0");

  /** factorial() computes n!, given a nonnegative BigInteger. 
   *  Receive:      n, a BigInteger
   *  Precondition: n >= 0
   *  Return:       n! = 1 * 2 * ... * (n-1) * n
   */

  public static BigInteger factorial(BigInteger n)
  {
    if(n.compareTo(ZERO) < 0)                     // check precondition
    {
      System.err.println("\n*** factorial(n): invalid argument "
                         + n + "received");
      System.exit(1);
    }
    final BigInteger ONE = new BigInteger("1");   // constant 1
    BigInteger product = new BigInteger("1");

    for (BigInteger count = new BigInteger("2");  // initExpression
                    count.compareTo(n) <= 0;      // booleanExpression
	              count = count.add(ONE))       // stepExpression
      product = product.multiply(count);          // statement

    return product;
  }

  public static void main(String [] args)
  {
    Screen theScreen = new Screen();
    Keyboard theKeyboard = new Keyboard();
    String numberString;
    BigInteger theNumber;
    for (;;)
    {
      theScreen.print("To compute n!, enter n (-1 to quit): ");
      numberString = theKeyboard.readWord();
	   theNumber = new BigInteger(numberString);

      if (theNumber.compareTo(ZERO) < 0) break;

      theScreen.println(theNumber + "! = "
           + factorial(theNumber) + "\n");
    }
  }
}

